#!/usr/local/bin/perl

=head1 NAME

giga2plain.pl

=head1 SYNOPSIS

giga2plain.pl is a document format converter that transforms GigaCorpus 
files into plain text.

=head1 DESCRIPTION

The program accepts input file(s) in the English GigaWord format and gives 
output in plain text

The English GigaWord format looks like:

 <DOC id="AFE19940512.0005" type="story" >
 <HEADLINE>
  This is a test
 </HEADLINE>
 <DATELINE>
 WASHINGTON, May 12 (AFP) 
 </DATELINE>
 <TEXT>
 <P>
 To demonstrate the options available we have incorporated
 in our description this example.
 </P>
 <P>
 We include punctutation ``examples'' such as the period, comma, 
 and quotations.
 </P>
 <P> 
 We also include some numbers to demonstrate how the 1 program 
 takes numbers 100 percent into consideration no matter if 
 you have 10.0 words or 100,000 words.
 </TEXT>
 </DOC>


The plain text output without defaults would look as follows:

  This is a test
  WASHINGTON , May 12 ( AFP ) 
  To demonstrate the options available we have incorporated in our description this example .
  We include punctutation " examples " such as the period , comma , and quotations . 
  We also include some numbers to demonstrate how the 1 program takes numbers 100 percent into consideration no matter if you have 10 . 0 words or 100 , 000 words.

   
Only the paragraphs, the dateline and the headline text is kept. The 
sentences between the paragraph tags are put on a single line. Punctuation 
is seperated from the words. The '' and `` quotations are replaced with " 
quotations

There are seven options that are implemented. 
1. --lowercase which converts all of the text to lower case including the 
    headline and the dateline

    this is a test
    washington , may 12 ( afp )
    to demonstrate ...

2. --removeheadline removes the headline from the text

    WASHINGTON , May 12 ( AFP ) 
    To demonstrate ...
    

3. --removedateline removes the dateline from the text    
    
    This is a test
    To demonstrate ...

4. --removenumbers removes all of the numbers from the text
    
    ...
    We also include some numbers to demonstrate how the program takes numbers percent into consideration no matter if you have words or words .
    ...

5. --removepunctuation removes all of the punctuation from the text

    ...
    We include punctutation examples such as the period comma and quotations
    ...

6. --replacenumbers replaces numbers with the NUMBER tag

    ...
    We also include some numbers to demonstrate how the NUMBER program takes numbers NUMBER percent into consideration no matter if you have NUMBER words or NUMBER words .
    ...

7. --replacepunctuation replaces the punctuation with the PUNCTUATION tag

    ...
    We include punctutation PUNCTUATION examples PUNCTUATION such as the period PUNCTUATION comma PUNCTUATION and quotations PUNCTUATION
    
=head1 AUTHOR

Bridget T McInnes, bthomson@cs.umn.edu

Ted Pedersen, tpederse@d.umn.edu

=head1 BUGS

=head1 SEE ALSO

 home page:    http://www.d.umn.edu/~tpederse/nsp.html

 mailing list: http://groups.yahoo.com/group/ngram/

=head1 COPYRIGHT

Copyright (C) 2005, Ted Pedersen and Bridget T McInnes

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

=cut

###############################################################################
#
#                       -------         CHANGELOG       ---------
#
#version        date            programmer      List of changes     change-id
#

###############################################################################

#-----------------------------------------------------------------------------
#                              Start of program
#-----------------------------------------------------------------------------

# we have to use commandline options, so use the necessary package!
use Getopt::Long;

# first check if no commandline options have been provided... in which case
# print out the usage notes!
if ( $#ARGV == -1 )
{
    &minimalUsageNotes();
    exit;
}

# now get the options!
GetOptions( "version", "help", "lowercase", "removeheadline", "removedateline", "removenumbers", "removepunctuation", "replacenumbers", "replacepunctuation");

# if help has been requested, print out help!
if ( defined $opt_help )
{
    $opt_help = 1;
    &showHelp();
    exit;
}

# if version has been requested, show version!
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

if( (defined $opt_removenumbers) && (defined $opt_replacenumbers) ) {
    
    print STDERR "Can not specify the removal and replacement of numbers.\n\n";
    $opt_help = 1;
    &showHelp();
    exit;
}

if( (defined $opt_removepunctuation) && (defined $opt_replacepunctuation) ) {
    
    print STDERR "Can not specify the removal and replacement of punctuation.\n\n";  
    $opt_help = 1;
    &showHelp();
    exit;
}   

&getSourceFiles(@ARGV);

# if not even one file found, complain and quit!
if ($#sourceFiles == -1)
{
    print STDERR "No input (SOURCE) file supplied!\n";
    askHelp();
    exit;
}

# having ascertained that we may open the destination file for output, lets 
# do so...



# now get the source files one by one from @sourceFiles, and process them in
# a loop!
foreach $source (@sourceFiles)
{
    # we already know that the file exists... that is checked by
    # &getSourceFiles, so no need to check it again! just open and
    # proceed.
    open( SRC, "$source" ) || die "Cant open SOURCE file $source, quitting";
    
    my $dest = $source . ".plain";
    
    open( DST, ">$dest");

    while(<SRC>) {
	
	chomp;
	
	#  if headline flag
	if($_=~/\<HEADLINE\>/) {
	    
	    my $line = <SRC>; chomp $line;
	    while(!($line =~/\<\/HEADLINE\>/)) {
		
		if(! (defined $opt_removeheadline) ) {
		    $line = process($line); print DST "$line\n";
		}
		$line = <SRC>; chomp $line;
	    }
	}
	
	#  if dateline
	elsif($_=~/\<DATELINE\>/) {
	
	    my $line = <SRC>; chomp $line;
	    while(!($line =~/\<\/DATELINE\>/)) {
		
		if(! (defined $opt_removedateline) ) {
		    $line = process($line); print DST "$line\n";
		}
		$line = <SRC>; chomp $line;
	    }
	}
	
	#  if paragraph 
	elsif($_=~/\<P\>/) {
	    
	    #  get the paragraph
	    my $line = <SRC>; chomp $line;
	    my $paragraph = "";
	    while(! ($line =~/\<\/P\>/) ) {
		$paragraph .= " " . $line;
		$line = <SRC>; chomp $line;
	    }

	    my $p_paragraph = process($paragraph);
	
	    $p_paragraph =~s/(PUNCTUATION )+/ PUNCTUATION /g;
	    $p_paragraph =~s/\s+/ /g;

	    print DST "$p_paragraph\n";
	}
	else {
	    # tag do nothing
	}
    }
    
    close(DST);
}

sub process
{
    my $line = shift;
    
    #  remove punctuation in acroynms
    my @array = split/\s+/, $line;
    
    foreach my $i (0..$#array) {
	
	#  checking for acronym 
	if($array[$i]=~/^[A-Z]\.([A-Z]\.)*$/) {
	    # leave as is ($array[$i]=~s/\.//g;)
	}
	
	#  check if a title
	elsif($array[$i]=~/(Mr\.|Mrs\.|Miss\.|Ms\.|Dr\.)/) {
	    # leave as is ($array[$i]=~s/\.//g;)
	}
	else {
	    $array[$i]=~s/\./ \. /g;
	}
    }    
    
    
    #  reset line
    $line = join " ", @array;

    #  seperate punctuation
    $line =~s/''/"/g;    $line =~s/``/"/g;
    $line =~s/([!@\#\$\%\^\&\*\?\"\:\;\(\)\[\],])\s/ $1 /g;
    $line =~s/--/ -- /g;
    $line =~s/\s+/ /g;

    #  put together funny hypens
    $line =~s/(\w+)-\s(\w+)/$1$2/g;
    
    #  lower case if user requested
    if( defined $opt_lowercase ) { $line = lc($line); }

    #  remove numbers if user requested
    if($opt_removenumbers) {
	$line=~s/[0-9]-?(\s,\s)?(\s\.\s)?(\s:\s)?//g;
    }

    #  replace numbers if user requested
    if($opt_replacenumbers) {
	$line=~s/[0-9]+(\s,\s[0-9]+)*(\s\.\s[0-9]+)*(\s:\s[0-9]+)*/NUMBER/g;
    }

    #  remove punctuation if user requested
    if($opt_removepunctuation) {
	$line =~s/([!@\#\$\%\^\&\*\?\"\:\;\(\)\[\],])//g;
	$line =~s/\s\.\s//g;
	$line =~s/\b(\w+) (\') (\w+)?\b/$1$3 /g;
    	$line =~s/\s--?\s/ /g;
	$line =~s/\s\'\s//g;
	$line =~s/\s\`\s//g;
    }

    #  replace punctuation if user requested
    if($opt_replacepunctuation) {
	
	#  combining the contractions
	$line =~s/\b(\w+) (\') (\w\w?)\b/$1$2$3 /g;
	$line =~s/\b(\w+)s\s(\')\s/$1s$2 /g;
	$line =~s/\s\'/ /g; $line =~s/\s\`/ /g;
	
	#  replacing punctuation with a marker
	$line =~s/([!@\#\$\%\^\&\*\?\"\:\;\(\)\[\],_])/ PUNCTUATION /g;
	$line =~s/\s\.\s/ PUNCTUATION /g;
	
	#  replacing -- with a punctuation marker
	$line =~s/\s--?\s/ PUNCTUATION /g;

	$line =~s/\s\'\s//g;
	$line =~s/\s\`\s//g;


    }
    
    $line =~s/\s+/ /g; $line=~s/^\s+//g; 

    return $line;

}
#########################################################################################
#  INPUT FUNCTIONS
#########################################################################################
# Function &getSourceFiles: function to take the command tail and
# return an array of text files to be used to count! while going thru the
# command line do the following processing:
#
# 1> if the string is a text file and can be opened, add it to the array.
# 2> if the string is a directory name, find all text files in that directory,
#    and append to array.
# 3> if the -r (recursive) option is set, go into all subdirectories of that
#    directory too, to do the above!

sub getSourceFiles
{
    # get the next commandline string...
    my $nextString = shift;
    $index = 0;
    
    while ( $nextString ) {

        if ( !( -e $nextString ) ) {
            # file doesn't exist... ignore!
	    $nextString = shift;
            next;
        }
	
    	if ( !( -r $nextString ) ) {
	    # file can't be read... ignore!
	    $nextString = shift;
            next;
    	}
    	
        if ( -d $nextString ) {
            # this is a directory, go and search this directory for text files
            &directorySearch( $nextString );
            $nextString = shift;
            next;
        }
	
        if ( !( -T $nextString ) ) {
            # file is not a text file... ignore!
	    $nextString = shift; 		
            next;
        }
	
        $sourceFiles[$index] = $nextString; 
        $index++;
        $nextString = shift;
    }
}

# function to (possibly recursively) search inside the given directory for
# text files
sub directorySearch
{
    my $directory = $_[0];
    
    opendir DIR, $directory || "Couldnt open directory $directory!\n";
    my @files = grep !/^\./, readdir DIR;
    @files = map "$directory/$_", @files;
    closedir DIR;
    
    my $file = "";
    
    foreach $file (@files)
    {
        if ( ( -d $file ) ) { &directorySearch($file); }
        if ( ( -T $file ) )
        {
            $sourceFiles[$index] = $file; 
            $index++;
        }
    }
}

# function to output a minimal usage note when the user has not provided any
# commandline options
sub minimalUsageNotes
{
    print STDERR "Usage: giga2plain.pl [OPTIONS] DESTINATION SOURCE [[, SOURCE] ...]\n";
    askHelp();
}

# function to output help messages for this program
sub showHelp
{
    print "Usage: giga2plain.pl [OPTIONS] DESTINATION SOURCE [[, SOURCE] ...]\n\n";
	  
    print "Converts one or more GigaWord files to plain text.\n\n";
	  
    print "OPTIONS:\n\n";
    
    print "  --version             Prints the version number.\n\n";
    
    print "  --help                Prints this help message.\n\n";

    print "  --lowercase           Converts all the text to lowercase letters.\n\n";
    
    print "  --removeheadline      Removes the headline information.\n\n";

    print "  --removedateline      Removes the dataline information.\n\n";

    print "  --removenumbers       Removes words consisting entirely of digits.\n\n";

    print "  --removepunctuation   Removes punctuation.\n\n";

    print "  --replacenumbers      Replaces numbers with the tag: NUMBER.\n\n";

    print "  --replacepunctuation  Replaces punctuation with the tag: PUNCTUATION.\n\n";

}

# function to output the version number
sub showVersion
{
    print STDERR "giga2plain.pl      -        version 0.2\n";
    print STDERR "Copyright (C) 2000-2003, Ted Pedersen & Bridget T McInnes\n";
    print STDERR "Date of Last Update 16/10/05\n";

}

# function to output "ask for help" message when the user's goofed up!
sub askHelp
{
    print STDERR "Type giga2plain.pl --help for help.\n";
}
